# Copyright (C) 2012, Aleksey Lim
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from gettext import gettext as _

import active_document as ad
enforce = ad.util.enforce

from restful_document import env
from restful_document.metadata import restful_method


class Document(ad.Document):
    """All RESTful document classes need to inherit this one."""

    @classmethod
    @restful_method(method='POST')
    def restful_post(cls):
        doc = cls.create(env.request.content)
        return {'guid': doc.guid}

    @classmethod
    @restful_method(method='GET')
    def restful_get(cls, **kwargs):
        offset = env.pop_int('offset', kwargs, None)
        limit = env.pop_int('limit', kwargs, None)
        query = env.pop_str('query', kwargs, None)
        reply = env.pop_list('reply', kwargs, None)
        order_by = env.pop_list('order_by', kwargs, None)

        documents, total = cls.find(offset, limit, kwargs, query, reply,
                order_by)
        return {'total': total.value,
                'documents': [i.all_properties(reply) for i in documents]}

    @restful_method(method='PUT')
    def restful_put(self, prop=None):
        if prop is None:
            self.update(self.guid, env.request.content)
        elif isinstance(self.metadata[prop], ad.BlobProperty):
            self.set_blob(prop, env.request.content_stream,
                    env.request.content_length)
        else:
            self[prop] = env.request.content
            self.post()

    @restful_method(method='DELETE')
    def restful_delete(self, prop=None):
        enforce(prop is None, env.Forbidden,
                _('Properties cannot be deleted'))
        self.delete(self.guid)

    @restful_method(method='GET')
    def restful_get_document(self, prop=None):
        if prop is None:
            reply = []
            for name, prop in self.metadata.items():
                if prop.is_trait:
                    reply.append(name)
            return self.all_properties(reply)
        elif isinstance(self.metadata[prop], ad.BlobProperty):
            env.responce['Content-Type'] = self.metadata[prop].mime_type
            return self.get_blob(prop)
        else:
            return self[prop]

    def all_properties(self, reply):
        result = {}
        for prop_name in (reply or ['guid']):
            result[prop_name] = self[prop_name]
        return result
