# Copyright (C) 2012, Aleksey Lim
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import json
import types
import logging
from gettext import gettext as _

import active_document as ad
enforce = ad.util.enforce
util = ad.util

from restful_document import env
from restful_document.metadata import Metadata


_logger = logging.getLogger('rd.router')


class Router(object):

    def __init__(self, classes):
        ad.init(classes)
        self.metadata = Metadata(classes)

        if 'SSH_ASKPASS' in os.environ:
            # Otherwise ssh-keygen will popup auth dialogs on registeration
            del os.environ['SSH_ASKPASS']

    def __call__(self, environ, start_response):
        env.request.set(environ)
        env.responce.set()

        _logger.debug('Processing request %s: %s',
                env.request.url, env.request.content)

        try:
            method = self.metadata.get_method()
            enforce(method is not None and \
                    method.method == env.request.method, env.BadRequest,
                    _('No way to handle the request'))
            result = method()

        except Exception, error:
            if isinstance(error, ad.Forbidden):
                util.exception()
                env.responce.status = env.Forbidden.status
                env.responce.update(env.Forbidden.headers)
            elif isinstance(error, env.HTTPError):
                env.responce.status = error.status
                env.responce.update(error.headers)
            else:
                util.exception(_('Error while processing "%s" request'),
                        env.request.url)
                env.responce.status = '500 Internal Server Error'

            env.responce['Content-Type'] = 'application/json'
            result = {'error': str(error), 'request': env.request.url}

        start_response(env.responce.status, env.responce.items())
        if isinstance(result, types.GeneratorType):
            for i in result:
                yield i
        else:
            if env.responce['Content-Type'] == 'application/json':
                result = json.dumps(result)
            yield result
